@@ -3,6 +3,9 @@ import threading
 from concurrent.futures import ThreadPoolExecutor, wait
 from typing import Optional
 
+from .sdk_flags import _SDKFlags
+from .utils import djb2_hash
+
 from .evaluation_details import EvaluationReason
 from .statsig_error_boundary import _StatsigErrorBoundary
 from .statsig_errors import StatsigValueError, StatsigNameError
@@ -17,22 +20,12 @@ IDLISTS_SYNC_INTERVAL = 60
 STORAGE_ADAPTER_KEY = "statsig.cache"
 SYNC_OUTDATED_MAX_S = 120
 
-
-def _is_specs_json_valid(specs_json):
-    if specs_json is None or specs_json.get("time") is None:
-        return False
-    if specs_json.get("has_updates", False) is False:
-        return False
-
-    return True
-
-
 class _SpecStore:
     _background_download_configs: Optional[threading.Thread]
     _background_download_id_lists: Optional[threading.Thread]
 
     def __init__(self, network: _StatsigNetwork, options: StatsigOptions, statsig_metadata: dict,
-                 error_boundary: _StatsigErrorBoundary, shutdown_event: threading.Event):
+                 error_boundary: _StatsigErrorBoundary, shutdown_event: threading.Event, sdk_key: str):
         self.last_update_time = 0
         self.initial_update_time = 0
         self.init_reason = EvaluationReason.uninitialized
@@ -46,20 +39,28 @@ class _SpecStore:
         self._background_download_configs = None
         self._background_download_id_lists = None
         self._sync_failure_count = 0
+        self._sdk_key = sdk_key
 
         self._configs = {}
         self._gates = {}
         self._layers = {}
         self._experiment_to_layer = {}
         self._sdk_keys_to_app_ids = {}
+        self._hashed_sdk_keys_to_app_ids = {}
 
         self._id_lists = {}
 
-    def initialize(self):
-        if self._options.local_mode:
-            self._initialized = True
-            return
+    def _is_specs_json_valid(self, specs_json):
+        if specs_json is None or specs_json.get("time") is None:
+            return False
+        hashed_sdk_key_used = specs_json.get("hashed_sdk_key_used", None)
+        if hashed_sdk_key_used is not None and hashed_sdk_key_used != djb2_hash(self._sdk_key):
+            return False
+        if specs_json.get("has_updates", False) is False:
+            return False
+        return True
 
+    def initialize(self):
         self._initialize_specs()
         self.initial_update_time = -1 if self.last_update_time == 0 else self.last_update_time
 
@@ -124,6 +125,9 @@ class _SpecStore:
     def get_target_app_for_sdk_key(self, sdk_key=None):
         if sdk_key is None:
             return None
+        target_app_id = self._hashed_sdk_keys_to_app_ids.get(djb2_hash(sdk_key))
+        if target_app_id is not None:
+            return target_app_id
         return self._sdk_keys_to_app_ids.get(sdk_key)
 
     def _initialize_specs(self):
@@ -145,7 +149,7 @@ class _SpecStore:
 
     def _process_specs(self, specs_json) -> bool:
         self._log_process("Processing specs...")
-        if not _is_specs_json_valid(specs_json):
+        if not self._is_specs_json_valid(specs_json):
             self._log_process("Failed to process specs")
             return False
 
@@ -169,11 +173,16 @@ class _SpecStore:
                 new_experiment_to_layer[experiment_name] = layer_name
 
         self._sdk_keys_to_app_ids = specs_json.get("sdk_keys_to_app_ids", {})
+        self._hashed_sdk_keys_to_app_ids = specs_json.get("hashed_sdk_keys_to_app_ids", {})
         self._gates = new_gates
         self._configs = new_configs
         self._layers = new_layers
         self._experiment_to_layer = new_experiment_to_layer
         self.last_update_time = specs_json.get("time", 0)
+
+        flags = specs_json.get("sdk_flags", {})
+        _SDKFlags.set_flags(flags)
+
         sampling_rate = specs_json.get("diagnostics", {})
         Diagnostics.set_sampling_rate(sampling_rate)
 
@@ -190,7 +199,7 @@ class _SpecStore:
 
         try:
             specs = json.loads(self._options.bootstrap_values)
-            if specs is None or not _is_specs_json_valid(specs):
+            if specs is None or not self._is_specs_json_valid(specs):
                 return
             if self._process_specs(specs):
                 self.init_reason = EvaluationReason.bootstrap
@@ -243,8 +252,6 @@ class _SpecStore:
     def download_config_spec_process(self, specs):
         try:
             Diagnostics.mark().download_config_specs().process().start()
-            if not _is_specs_json_valid(specs):
-                return
 
             self._log_process("Done loading specs")
             if self._process_specs(specs):
@@ -257,7 +264,7 @@ class _SpecStore:
                 {'success': self.init_reason == EvaluationReason.network})
 
     def _save_to_storage_adapter(self, specs):
-        if not _is_specs_json_valid(specs):
+        if not self._is_specs_json_valid(specs):
             return
 
         if self._options.data_store is None:
@@ -282,7 +289,6 @@ class _SpecStore:
             globals.logger.warning(
                 "Invalid type returned from StatsigOptions.data_store")
             return
-
         adapter_time = cache.get("time", None)
         if not isinstance(adapter_time,
                           int) or adapter_time < self.last_update_time:
@@ -381,6 +387,7 @@ class _SpecStore:
             self._error_boundary.log_exception("_download_id_lists_process", e)
         finally:
             Diagnostics.mark().get_id_list_sources().process().end({'success': not threw_error})
+
     def _download_single_id_list(
             self, url, list_name, local_list, all_lists, start_index):
         resp = self._network.get_request(
