@@ -107,7 +107,7 @@ def get_experiment(user: StatsigUser, experiment: str) -> DynamicConfig:
     :param experiment: The name of the experiment
     :return: A DynamicConfig object
     """
-    return get_config(user, experiment)
+    return __instance.get_experiment(user, experiment)
 
 
 def get_experiment_with_exposure_logging_disabled(user: StatsigUser, experiment: str) -> DynamicConfig:
@@ -118,7 +118,7 @@ def get_experiment_with_exposure_logging_disabled(user: StatsigUser, experiment:
     :param experiment: The name of the experiment
     :return: A DynamicConfig object
     """
-    return get_config_with_exposure_logging_disabled(user, experiment)
+    return __instance.get_experiment(user, experiment, log_exposure=False)
 
 
 def manually_log_experiment_exposure(user: StatsigUser, experiment: str):
@@ -207,6 +207,18 @@ def override_experiment(experiment: str, value: object,
     __instance.override_experiment(experiment, value, user_id)
 
 
+def override_layer(layer: str, value: object,
+                   user_id: Optional[str] = None):
+    """
+    Override the Layer value for the given user
+
+    :param layer: The name of the layer being overriden
+    :param value: The value to override the layer with
+    :param user_id: (Optional) The user_id of the user to override
+    """
+    __instance.override_layer(layer, value, user_id)
+
+
 def remove_gate_override(gate: str, user_id: Optional[str] = None):
     """
     Remove the overriden value of a Feature Gate for a given user
@@ -237,6 +249,16 @@ def remove_experiment_override(experiment: str, user_id: Optional[str] = None):
     __instance.remove_experiment_override(experiment, user_id)
 
 
+def remove_layer_override(layer: str, user_id: Optional[str] = None):
+    """
+    Remove the overriden value of a Layer for a given user
+
+    :param layer: The name of the layer that was overriden
+    :param user_id: (Optional) The user_id of the user to remove override for
+    """
+    __instance.remove_layer_override(layer, user_id)
+
+
 def remove_all_overrides():
     """
     Removes all overrides for all users
